// Author: Marcelo Zimbres Silva <mailto:mzimbres@gmail.com>

/* Copyright (C) 2010 Marcelo Zimbres Silva
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

// ROOT

#include "TMath.h"
#include "TVirtualFFT.h"
#include "TArrayD.h"

// SWAT

#include "TWignerd.h"
#include "TSmallWignerd.h"

ClassImp(TSmallWignerd);

using namespace TMath;

 ///////////////////////////////////////////////////////////////////////
 //                                                                   //
 // Calculate small wigner-d functions using FFT.                     //
 //                                                                   //
 ///////////////////////////////////////////////////////////////////////


//_____________________________________________________________________
TSmallWignerd::TSmallWignerd(Int_t size): fSize(size)
{
   // DCT and DST

   //Int_t even = 2;
   //Int_t odd = 6;

   //fEven = TVirtualFFT::SineCosine(1,&size,&even,"M K");
   //fOdd = TVirtualFFT::SineCosine(1,&size,&odd,"M K");

   fEven = TVirtualFFT::FFT(1,&size,"C2R M K");
   fOdd = TVirtualFFT::FFT(1,&size,"C2R M K");
}

//_____________________________________________________________________
TSmallWignerd::~TSmallWignerd()
{
   // DCT and DST

   delete fEven;
   delete fOdd;
}

//_____________________________________________________________________
Double_t* TSmallWignerd::Get(const TWignerd& wig,Int_t m,Int_t n) const
{
   // Wigner d functions.

   Int_t l = wig.GetL();
   TArrayD fRe(fSize);
   TArrayD fIm(fSize);

   if (TMath::Even(m-n)){

      for (Int_t u = 0; u <= l; ++u)
         fRe[u] = wig.Delta(u,m)*wig.Delta(u,n);

      if ((TMath::Abs(n - m) & 3))
         for (Int_t i = 0; i <= l; ++i)
            fRe[i] = -fRe[i];
 
      fEven->SetPointsComplex(fRe.GetArray(),fIm.GetArray());
      fEven->Transform();
      return fEven->GetPointsReal();
   } else {

      for (Int_t u = 0; u <= l; ++u)
         fIm[u] = wig.Delta(u,m)*wig.Delta(u,n);

      if ((TMath::Abs(n - m + 1) & 3))
         for (Int_t i = 0; i <= l; ++i)
            fIm[i] = -fIm[i];
      
      fOdd->SetPointsComplex(fRe.GetArray(),fIm.GetArray());
      fOdd->Transform();
      return fOdd->GetPointsReal();
   } 
}


